@@ -4,6 +4,7 @@ Create ssh executor system
 '''
 # Import python libs
 from __future__ import print_function
+from __future__ import absolute_import
 import copy
 import getpass
 import json
@@ -19,13 +20,14 @@ import yaml
 import uuid
 import tempfile
 import binascii
+from salt.ext.six.moves import input
 
 # Import salt libs
 import salt.client.ssh.shell
 import salt.client.ssh.wrapper
 import salt.config
 import salt.exceptions
-import salt.exitcodes
+import salt.defaults.exitcodes
 import salt.log
 import salt.loader
 import salt.minion
@@ -38,7 +40,7 @@ import salt.utils.atomicfile
 import salt.utils.thin
 import salt.utils.verify
 import salt.utils.network
-from salt._compat import string_types
+from salt.ext.six import string_types
 from salt.utils import is_windows
 
 try:
@@ -132,7 +134,7 @@ done
 echo "ERROR: Unable to locate appropriate python command" >&2
 exit $EX_PYTHON_OLD
 EOF'''.format(
-    EX_THIN_PYTHON_OLD=salt.exitcodes.EX_THIN_PYTHON_OLD,
+    EX_THIN_PYTHON_OLD=salt.defaults.exitcodes.EX_THIN_PYTHON_OLD,
 )
 
 if not is_windows():
@@ -162,21 +164,35 @@ class SSH(object):
         else:
             self.event = None
         self.opts = opts
+        if not salt.utils.which('ssh'):
+            raise salt.exceptions.SaltSystemExit('No ssh binary found in path -- ssh must be installed for salt-ssh to run. Exiting.')
         self.opts['_ssh_version'] = ssh_version()
         self.tgt_type = self.opts['selected_target_option'] \
                 if self.opts['selected_target_option'] else 'glob'
-        self.roster = salt.roster.Roster(opts, opts.get('roster'))
+        self.roster = salt.roster.Roster(opts, opts.get('roster', 'flat'))
         self.targets = self.roster.targets(
                 self.opts['tgt'],
                 self.tgt_type)
-        priv = self.opts.get(
-                'ssh_priv',
-                os.path.join(
-                    self.opts['pki_dir'],
-                    'ssh',
-                    'salt-ssh.rsa'
+        # If we're in a wfunc, we need to get the ssh key location from the
+        # top level opts, stored in __master_opts__
+        if '__master_opts__' in self.opts:
+            priv = self.opts['__master_opts__'].get(
+                    'ssh_priv',
+                    os.path.join(
+                        self.opts['__master_opts__']['pki_dir'],
+                        'ssh',
+                        'salt-ssh.rsa'
+                        )
+                    )
+        else:
+            priv = self.opts.get(
+                    'ssh_priv',
+                    os.path.join(
+                        self.opts['pki_dir'],
+                        'ssh',
+                        'salt-ssh.rsa'
+                        )
                     )
-                )
         if not os.path.isfile(priv):
             try:
                 salt.client.ssh.shell.gen_key(priv)
@@ -239,18 +255,17 @@ class SSH(object):
         '''
         Deploy the SSH key if the minions don't auth
         '''
-        if not isinstance(ret[host], dict):
-            if self.opts.get('ssh_key_deploy'):
-                target = self.targets[host]
-                if 'passwd' in target:
-                    self._key_deploy_run(host, target, False)
+        if not isinstance(ret[host], dict) or self.opts.get('ssh_key_deploy'):
+            target = self.targets[host]
+            if 'passwd' in target or self.opts['ssh_passwd']:
+                self._key_deploy_run(host, target, False)
             return ret
         if ret[host].get('stderr', '').count('Permission denied'):
             target = self.targets[host]
             # permission denied, attempt to auto deploy ssh key
             print(('Permission denied for host {0}, do you want to deploy '
                    'the salt-ssh key? (password required):').format(host))
-            deploy = raw_input('[Y/n] ')
+            deploy = input('[Y/n] ')
             if deploy.startswith(('n', 'N')):
                 return ret
             target['passwd'] = getpass.getpass(
@@ -300,11 +315,11 @@ class SSH(object):
                 if stderr:
                     return {host: stderr}
                 return {host: 'Bad Return'}
-        if salt.exitcodes.EX_OK != retcode:
+        if salt.defaults.exitcodes.EX_OK != retcode:
             return {host: stderr}
         return {host: stdout}
 
-    def handle_routine(self, que, opts, host, target):
+    def handle_routine(self, que, opts, host, target, mine=False):
         '''
         Run the routine in a "Thread", put a dict on the queue
         '''
@@ -316,6 +331,7 @@ class SSH(object):
                 mods=self.mods,
                 fsclient=self.fsclient,
                 thin=self.thin,
+                mine=mine,
                 **target)
         ret = {'id': single.id}
         stdout, stderr, retcode = single.run()
@@ -338,7 +354,7 @@ class SSH(object):
             }
         que.put(ret)
 
-    def handle_ssh(self):
+    def handle_ssh(self, mine=False):
         '''
         Spin up the needed threads or processes and execute the subsequent
         routines
@@ -366,6 +382,7 @@ class SSH(object):
                         self.opts,
                         host,
                         self.targets[host],
+                        mine,
                         )
                 routine = multiprocessing.Process(
                                 target=self.handle_routine,
@@ -378,41 +395,49 @@ class SSH(object):
                 ret = que.get(False)
                 if 'id' in ret:
                     returned.add(ret['id'])
+                    yield {ret['id']: ret['ret']}
             except Exception:
                 pass
             for host in running:
-                if host in returned:
-                    if not running[host]['thread'].is_alive():
-                        running[host]['thread'].join()
-                        rets.add(host)
+                if not running[host]['thread'].is_alive():
+                    if host not in returned:
+                        # Try to get any returns that came through since we
+                        # last checked
+                        try:
+                            while True:
+                                ret = que.get(False)
+                                if 'id' in ret:
+                                    returned.add(ret['id'])
+                                    yield {ret['id']: ret['ret']}
+                        except Exception:
+                            pass
+
+                        if host not in returned:
+                            error = ('Target \'{0}\' did not return any data, '
+                                     'probably due to an error.').format(host)
+                            ret = {'id': host,
+                                   'ret': error}
+                            log.error(error)
+                            yield {ret['id']: ret['ret']}
+                    running[host]['thread'].join()
+                    rets.add(host)
             for host in rets:
                 if host in running:
                     running.pop(host)
-            if ret:
-                if not isinstance(ret, dict):
-                    continue
-                yield {ret['id']: ret['ret']}
             if len(rets) >= len(self.targets):
                 break
+            # Sleep when limit or all threads started
+            if len(running) >= self.opts.get('ssh_max_procs', 25) or len(self.targets) >= len(running):
+                time.sleep(0.1)
 
-    def run_iter(self):
+    def run_iter(self, mine=False):
         '''
         Execute and yield returns as they come in, do not print to the display
-        '''
-        for ret in self.handle_ssh():
-            yield ret
 
-    def cache_job(self, jid, id_, ret):
-        '''
-        Cache the job information
-        '''
-        self.returners['{0}.returner'.format(self.opts['master_job_cache'])]({'jid': jid,
-                                                                                      'id': id_,
-                                                                                      'return': ret})
-
-    def run(self):
-        '''
-        Execute the overall routine
+        mine
+            The Single objects will use mine_functions defined in the roster,
+            pillar, or master config (they will be checked in that order) and
+            will modify the argv with the arguments from mine_functions
         '''
         fstr = '{0}.prep_jid'.format(self.opts['master_job_cache'])
         jid = self.returners[fstr]()
@@ -420,7 +445,7 @@ class SSH(object):
         # Save the invocation information
         argv = self.opts['argv']
 
-        if self.opts['raw_shell']:
+        if self.opts.get('raw_shell', False):
             fun = 'ssh._raw'
             args = argv
         else:
@@ -439,6 +464,58 @@ class SSH(object):
         # save load to the master job cache
         self.returners['{0}.save_load'.format(self.opts['master_job_cache'])](jid, job_load)
 
+        for ret in self.handle_ssh(mine=mine):
+            host = next(ret.iterkeys())
+            self.cache_job(jid, host, ret[host], fun)
+            if self.event:
+                self.event.fire_event(
+                        ret,
+                        salt.utils.event.tagify(
+                            [jid, 'ret', host],
+                            'job'))
+            yield ret
+
+    def cache_job(self, jid, id_, ret, fun):
+        '''
+        Cache the job information
+        '''
+        self.returners['{0}.returner'.format(self.opts['master_job_cache'])]({'jid': jid,
+                                                                              'id': id_,
+                                                                              'return': ret,
+                                                                              'fun': fun})
+
+    def run(self):
+        '''
+        Execute the overall routine, print results via outputters
+        '''
+        fstr = '{0}.prep_jid'.format(self.opts['master_job_cache'])
+        jid = self.returners[fstr]()
+
+        # Save the invocation information
+        argv = self.opts['argv']
+
+        if self.opts.get('raw_shell', False):
+            fun = 'ssh._raw'
+            args = argv
+        else:
+            fun = argv[0] if argv else ''
+            args = argv[1:]
+
+        job_load = {
+            'jid': jid,
+            'tgt_type': self.tgt_type,
+            'tgt': self.opts['tgt'],
+            'user': self.opts['user'],
+            'fun': fun,
+            'arg': args,
+            }
+
+        # save load to the master job cache
+        try:
+            self.returners['{0}.save_load'.format(self.opts['master_job_cache'])](jid, job_load)
+        except Exception as exc:
+            log.error('Could not save load with returner {0}: {1}'.format(self.opts['master_job_cache'], exc))
+
         if self.opts.get('verbose'):
             msg = 'Executing job with jid {0}'.format(jid)
             print(msg)
@@ -447,8 +524,8 @@ class SSH(object):
         sret = {}
         outputter = self.opts.get('output', 'nested')
         for ret in self.handle_ssh():
-            host = ret.iterkeys().next()
-            self.cache_job(jid, host, ret[host])
+            host = next(ret.iterkeys())
+            self.cache_job(jid, host, ret[host], fun)
             ret = self.key_deploy(host, ret)
             if not isinstance(ret[host], dict):
                 p_data = {host: ret[host]}
@@ -495,13 +572,19 @@ class Single(object):
             port=None,
             passwd=None,
             priv=None,
-            timeout=None,
+            timeout=30,
             sudo=False,
             tty=False,
             mods=None,
             fsclient=None,
             thin=None,
+            mine=False,
+            minion_opts=None,
             **kwargs):
+        # Get mine setting and mine_functions if defined in kwargs (from roster)
+        self.mine = mine
+        self.mine_functions = kwargs.get('mine_functions')
+
         self.opts = opts
         self.tty = tty
         if kwargs.get('wipe'):
@@ -543,12 +626,16 @@ class Single(object):
                 'sudo': sudo,
                 'tty': tty,
                 'mods': self.mods}
-        self.minion_config = yaml.dump(
-                {
+        self.minion_opts = opts.get('ssh_minion_opts', {})
+        if minion_opts is not None:
+            self.minion_opts.update(minion_opts)
+        self.minion_opts.update({
                     'root_dir': os.path.join(self.thin_dir, 'running_data'),
                     'id': self.id,
                     'sock_dir': '/',
-                }, width=1000).strip()
+                    'log_file': 'salt-call.log'
+                })
+        self.minion_config = yaml.dump(self.minion_opts)
         self.target = kwargs
         self.target.update(args)
         self.serial = salt.payload.Serial(opts)
@@ -561,23 +648,9 @@ class Single(object):
         Return the function name and the arg list
         '''
         fun = self.argv[0] if self.argv else ''
-        args = []
-        kws = {}
-        for arg in self.argv[1:]:
-            # FIXME - there is a bug here that will steal a non-keyword argument.
-            # example:
-            #
-            # .. code-block:: bash
-            #
-            #     salt-ssh '*' cmd.run_all 'n=$((RANDOM%8)); exit $n'
-            #
-            # The 'n=' appears to be a keyword argument, but it is
-            # simply the argument!
-            if re.match(r'\w+=', arg):
-                (key, val) = arg.split('=', 1)
-                kws[key] = val
-            else:
-                args.append(arg)
+        parsed = salt.utils.args.parse_input(self.argv[1:], condition=False)
+        args = parsed[0]
+        kws = parsed[1]
         return fun, args, kws
 
     def _escape_arg(self, arg):
@@ -626,11 +699,11 @@ class Single(object):
         '''
         stdout = stderr = retcode = None
 
-        if self.opts.get('raw_shell'):
+        if self.opts.get('raw_shell', False):
             cmd_str = ' '.join([self._escape_arg(arg) for arg in self.argv])
             stdout, stderr, retcode = self.shell.exec_cmd(cmd_str)
 
-        elif self.fun in self.wfuncs:
+        elif self.fun in self.wfuncs or self.mine:
             stdout = self.run_wfunc()
 
         else:
@@ -675,13 +748,15 @@ class Single(object):
                 self.opts,
                 self.id,
                 fsclient=self.fsclient,
+                minion_opts=self.minion_opts,
                 **self.target)
-            opts_pkg = pre_wrapper['test.opts_pkg']()
+            opts_pkg = pre_wrapper['test.opts_pkg']()  # pylint: disable=E1102
             opts_pkg['file_roots'] = self.opts['file_roots']
             opts_pkg['pillar_roots'] = self.opts['pillar_roots']
             opts_pkg['ext_pillar'] = self.opts['ext_pillar']
             opts_pkg['extension_modules'] = self.opts['extension_modules']
             opts_pkg['_ssh_version'] = self.opts['_ssh_version']
+            opts_pkg['__master_opts__'] = self.context['master_opts']
             if '_caller_cachedir' in self.opts:
                 opts_pkg['_caller_cachedir'] = self.opts['_caller_cachedir']
             else:
@@ -691,7 +766,7 @@ class Single(object):
 
             if '_error' in opts_pkg:
                 #Refresh failed
-                ret = json.dumps({'local': opts_pkg})
+                ret = json.dumps({'local': opts_pkg['_error']})
                 return ret
 
             pillar = salt.pillar.Pillar(
@@ -731,11 +806,35 @@ class Single(object):
             opts,
             self.id,
             fsclient=self.fsclient,
+            minion_opts=self.minion_opts,
             **self.target)
         self.wfuncs = salt.loader.ssh_wrapper(opts, wrapper, self.context)
         wrapper.wfuncs = self.wfuncs
+
+        # We're running in the mind, need to fetch the arguments from the
+        # roster, pillar, master config (in that order)
+        if self.mine:
+            mine_args = None
+            if self.mine_functions and self.fun in self.mine_functions:
+                mine_args = self.mine_functions[self.fun]
+            elif opts['pillar'] and self.fun in opts['pillar'].get('mine_functions', {}):
+                mine_args = opts['pillar']['mine_functions'][self.fun]
+            elif self.fun in self.context['master_opts'].get('mine_functions', {}):
+                mine_args = self.context['master_opts']['mine_functions'][self.fun]
+
+            # If we found mine_args, replace our command's args
+            if isinstance(mine_args, dict):
+                self.args = []
+                self.kwargs = mine_args
+            elif isinstance(mine_args, list):
+                self.args = mine_args
+                self.kwargs = {}
+
         try:
-            result = self.wfuncs[self.fun](*self.args, **self.kwargs)
+            if self.mine:
+                result = wrapper[self.fun](*self.args, **self.kwargs)
+            else:
+                result = self.wfuncs[self.fun](*self.args, **self.kwargs)
         except TypeError as exc:
             result = 'TypeError encountered executing {0}: {1}'.format(self.fun, exc)
         except Exception as exc:
@@ -759,11 +858,16 @@ class Single(object):
             cachedir = self.opts['cachedir']
         thin_sum = salt.utils.thin.thin_sum(cachedir, 'sha1')
         debug = ''
-        if salt.log.LOG_LEVELS['debug'] >= salt.log.LOG_LEVELS[self.opts['log_level']]:
+        if not self.opts.get('log_level'):
+            self.opts['log_level'] = 'info'
+        if salt.log.LOG_LEVELS['debug'] >= salt.log.LOG_LEVELS[self.opts.get('log_level', 'info')]:
             debug = '1'
         arg_str = '''
 OPTIONS = OBJ()
-OPTIONS.config = '{0}'
+OPTIONS.config = \
+"""
+{0}
+"""
 OPTIONS.delimiter = '{1}'
 OPTIONS.saltdir = '{2}'
 OPTIONS.checksum = '{3}'
@@ -777,7 +881,7 @@ ARGS = {9}\n'''.format(self.minion_config,
                          self.thin_dir,
                          thin_sum,
                          'sha1',
-                         salt.__version__,
+                         salt.version.__version__,
                          self.mods.get('version', ''),
                          self.wipe,
                          self.tty,
@@ -793,23 +897,6 @@ ARGS = {9}\n'''.format(self.minion_config,
 
         return cmd
 
-    def cmd(self):
-        '''
-        Prepare the pre-check command to send to the subsystem
-        '''
-        if self.fun.startswith('state.highstate'):
-            self.highstate_seed()
-        elif self.fun.startswith('state.sls'):
-            args, kwargs = salt.minion.load_args_and_kwargs(
-                self.sls_seed,
-                salt.utils.args.parse_input(self.args)
-            )
-            self.sls_seed(*args, **kwargs)
-        cmd_str = self._cmd_str()
-
-        for stdout, stderr, retcode in self.shell.exec_nb_cmd(cmd_str):
-            yield stdout, stderr, retcode
-
     def shim_cmd(self, cmd_str):
         '''
         Run a shim command.
@@ -841,30 +928,31 @@ ARGS = {9}\n'''.format(self.minion_config,
             pass
 
         # Execute shim
-        ret = self.shell.exec_cmd('/bin/sh $HOME/{0}'.format(target_shim_file))
+        ret = self.shell.exec_cmd('/bin/sh \'$HOME/{0}\''.format(target_shim_file))
 
         # Remove shim from target system
-        self.shell.exec_cmd('rm $HOME/{0}'.format(target_shim_file))
+        self.shell.exec_cmd('rm \'$HOME/{0}\''.format(target_shim_file))
 
         return ret
 
     def cmd_block(self, is_retry=False):
         '''
         Prepare the pre-check command to send to the subsystem
-        '''
-        # 1. execute SHIM + command
-        # 2. check if SHIM returns a master request or if it completed
-        # 3. handle any master request
-        # 4. re-execute SHIM + command
-        # 5. split SHIM results from command results
-        # 6. return command results
 
+        1. execute SHIM + command
+        2. check if SHIM returns a master request or if it completed
+        3. handle any master request
+        4. re-execute SHIM + command
+        5. split SHIM results from command results
+        6. return command results
+        '''
+        self.argv = _convert_args(self.argv)
         log.debug('Performing shimmed, blocking command as follows:\n{0}'.format(' '.join(self.argv)))
         cmd_str = self._cmd_str()
         stdout, stderr, retcode = self.shim_cmd(cmd_str)
 
-        log.debug('STDOUT {1}\n{0}'.format(stdout, self.target['host']))
-        log.debug('STDERR {1}\n{0}'.format(stderr, self.target['host']))
+        log.trace('STDOUT {1}\n{0}'.format(stdout, self.target['host']))
+        log.trace('STDERR {1}\n{0}'.format(stderr, self.target['host']))
         log.debug('RETCODE {1}: {0}'.format(retcode, self.target['host']))
 
         error = self.categorize_shim_errors(stdout, stderr, retcode)
@@ -898,7 +986,8 @@ ARGS = {9}\n'''.format(self.minion_config,
             # RSTR was found in stdout but not stderr - which means there
             # is a SHIM command for the master.
             shim_command = re.split(r'\r?\n', stdout, 1)[0].strip()
-            if 'deploy' == shim_command and retcode == salt.exitcodes.EX_THIN_DEPLOY:
+            log.debug('SHIM retcode({0}) and command: {1}'.format(retcode, shim_command))
+            if 'deploy' == shim_command and retcode == salt.defaults.exitcodes.EX_THIN_DEPLOY:
                 self.deploy()
                 stdout, stderr, retcode = self.shim_cmd(cmd_str)
                 if not re.search(RSTR_RE, stdout) or not re.search(RSTR_RE, stderr):
@@ -952,17 +1041,22 @@ ARGS = {9}\n'''.format(self.minion_config,
                 'sudo expected a password, NOPASSWD required'
             ),
             (
-                (salt.exitcodes.EX_THIN_PYTHON_OLD,),
+                (salt.defaults.exitcodes.EX_THIN_PYTHON_OLD,),
                 'Python interpreter is too old',
                 'salt requires python 2.6 or newer on target hosts'
             ),
             (
-                (salt.exitcodes.EX_THIN_CHECKSUM,),
+                (salt.defaults.exitcodes.EX_THIN_CHECKSUM,),
                 'checksum mismatched',
                 'The salt thin transfer was corrupted'
             ),
             (
-                (salt.exitcodes.EX_CANTCREAT,),
+                (salt.defaults.exitcodes.EX_SCP_NOT_FOUND,),
+                'scp not found',
+                'No scp binary. openssh-clients package required'
+            ),
+            (
+                (salt.defaults.exitcodes.EX_CANTCREAT,),
                 'salt path .* exists but is not a directory',
                 'A necessary path for salt thin unexpectedly exists:\n ' + stderr,
             ),
@@ -987,7 +1081,7 @@ ARGS = {9}\n'''.format(self.minion_config,
                 perm_error_fmt.format(stderr)
             ),
             (
-                (salt.exitcodes.EX_SOFTWARE,),
+                (salt.defaults.exitcodes.EX_SOFTWARE,),
                 'exists but is not',
                 'An internal error occurred with the shim, please investigate:\n ' + stderr,
             ),
@@ -1122,3 +1216,20 @@ def ssh_version():
         return ret[1].split(',')[0].split('_')[1]
     except IndexError:
         return '2.0'
+
+
+def _convert_args(args):
+    '''
+    Take a list of args, and convert any dicts inside the list to keyword
+    args in the form of `key=value`, ready to be passed to salt-ssh
+    '''
+    converted = []
+    for arg in args:
+        if isinstance(arg, dict):
+            for key in list(arg.keys()):
+                if key == '__kwarg__':
+                    continue
+                converted.append('{0}={1}'.format(key, arg[key]))
+        else:
+            converted.append(arg)
+    return converted
