@@ -2,6 +2,12 @@
 '''
 A module to manage software on Windows
 
+.. important::
+    If you feel that Salt should be using this module to manage packages on a
+    minion, and it is using a different module (or gives an error similar to
+    *'pkg.install' is not available*), see :ref:`here
+    <module-provider-override>`.
+
 :depends:   - win32com
             - win32con
             - win32api
@@ -17,7 +23,6 @@ except ImportError:
     HAS_DEPENDENCIES = False
 
 # Import python libs
-import copy
 import logging
 try:
     import msgpack
@@ -26,10 +31,10 @@ except ImportError:
 import os
 import locale
 from distutils.version import LooseVersion  # pylint: disable=E0611
-import re
 
 # Import salt libs
 import salt.utils
+from salt.exceptions import MinionError
 
 log = logging.getLogger(__name__)
 
@@ -92,7 +97,10 @@ def latest_version(*names, **kwargs):
         # get latest available (from win_repo) version of package
         pkg_info = _get_package_info(name)
         log.trace('Raw win_repo pkg_info for {0} is {1}'.format(name, pkg_info))
-        latest_available = _get_latest_pkg_version(pkg_info)
+        if not pkg_info:
+            latest_available = 'Not Found in WinRepo (names are case sensitive)'
+        else:
+            latest_available = _get_latest_pkg_version(pkg_info)
         if latest_available:
             log.debug('Latest available version of package {0} is {1}'.format(name, latest_available))
 
@@ -195,17 +203,7 @@ def version(*names, **kwargs):
         return ''
     if len(names) > 1:
         reverse_dict = {}
-        for name in names:
-            ret[name] = ''
-            versions = _get_package_info(name)
-            if versions:
-                for val in versions.itervalues():
-                    if 'full_name' in val and len(val.get('full_name', '')) > 0:
-                        reverse_dict[val.get('full_name', '')] = name
-                        win_names.append(val.get('full_name', ''))
-            else:
-                win_names.append(name)
-        nums = __salt__['pkg_resource.version'](*win_names, **kwargs)
+        nums = __salt__['pkg_resource.version'](*names, **kwargs)
         if len(nums):
             for num, val in nums.iteritems():
                 if len(val) > 0:
@@ -237,14 +235,6 @@ def list_pkgs(versions_as_list=False, **kwargs):
             for x in ('removed', 'purge_desired')]):
         return {}
 
-    if 'pkg.list_pkgs' in __context__:
-        if versions_as_list:
-            return __context__['pkg.list_pkgs']
-        else:
-            ret = copy.deepcopy(__context__['pkg.list_pkgs'])
-            __salt__['pkg_resource.stringify'](ret)
-            return ret
-
     ret = {}
     name_map = _get_name_map()
     with salt.utils.winapi.Com():
@@ -252,13 +242,8 @@ def list_pkgs(versions_as_list=False, **kwargs):
             if key in name_map:
                 key = name_map[key]
             __salt__['pkg_resource.add_pkg'](ret, key, val)
-        for key, val in _get_msi_software().iteritems():
-            if key in name_map:
-                key = name_map[key]
-            __salt__['pkg_resource.add_pkg'](ret, key, val)
 
     __salt__['pkg_resource.sort_pkglist'](ret)
-    __context__['pkg.list_pkgs'] = copy.deepcopy(ret)
     if not versions_as_list:
         __salt__['pkg_resource.stringify'](ret)
     return ret
@@ -271,9 +256,7 @@ def _search_software(target):
     values added to the list passed in
     '''
     search_results = {}
-    software = dict(
-        list(_get_reg_software().items()) +
-        list(_get_msi_software().items()))
+    software = dict(_get_reg_software().items())
     for key, value in software.items():
         if key is not None:
             if target.lower() in key.lower():
@@ -281,56 +264,6 @@ def _search_software(target):
     return search_results
 
 
-def _get_msi_software():
-    '''
-    Uses powershell to search the msi product databases, returns a
-    dict keyed on the product name as the key and the version as the
-    value. If powershell is not available, returns `{}`
-    '''
-    win32_products = {}
-
-    # Don't use WMI to select from `Win32_product`, that has nasty
-    # side effects. Use the `WindowsInstaller.Installer` COM object's
-    # `ProductsEx`. Jumping through powershell because `ProductsEx` is
-    # a get property that takes 3 arguments, and `win32com` can't call
-    # that
-    #
-    # see https://github.com/saltstack/salt/issues/12550 for detail
-
-    # powershell script to fetch (name, version) from COM, and write
-    # without word-wrapping. Attempting to target minimal powershell
-    # versions
-    ps = '''
-$msi = New-Object -ComObject WindowsInstaller.Installer;
-$msi.GetType().InvokeMember('ProductsEx', 'GetProperty', $null, $msi, ('', 's-1-1-0', 7))
-| select @{
-      name='name';
-      expression={$_.GetType().InvokeMember('InstallProperty', 'GetProperty', $null, $_, ('ProductName'))}
-    },
-    @{
-      name='version';
-      expression={$_.GetType().InvokeMember('InstallProperty', 'GetProperty', $null, $_, ('VersionString'))}
-    }
-| Write-host
-'''.replace('\n', ' ')  # make this a one-liner
-
-    ret = __salt__['cmd.run_all'](ps, shell='powershell', python_shell=True)
-    # sometimes the powershell reflection fails on a single product,
-    # giving us a non-zero return code AND useful output. Ignore RC
-    # and just try to process stdout, which should empty if the cmd
-    # failed.
-    #
-    # each line of output looks like:
-    #
-    # `@{name=PRD_NAME; version=PRD_VER}`
-    pattern = r'@{name=(.+); version=(.+)}'
-    for m in re.finditer(pattern, ret['stdout']):
-        (prd_name, prd_ver) = m.groups()
-        win32_products[prd_name] = prd_ver
-
-    return win32_products
-
-
 def _get_reg_software():
     '''
     This searches the uninstall keys in the registry to find
@@ -356,8 +289,7 @@ def _get_reg_software():
 
     #attempt to corral the wild west of the multiple ways to install
     #software in windows
-    reg_entries = dict(list(_get_user_keys().items()) +
-                       list(_get_machine_keys().items()))
+    reg_entries = dict(_get_machine_keys().items())
     for reg_hive, reg_keys in reg_entries.items():
         for reg_key in reg_keys:
             try:
@@ -376,8 +308,6 @@ def _get_reg_software():
                     reg_hive,
                     prd_uninst_key,
                     'WindowsInstaller')
-                if windows_installer != 'Not Found' and windows_installer:
-                    continue
 
                 prd_name = _get_reg_value(
                     reg_hive,
@@ -416,39 +346,6 @@ def _get_machine_keys():
     return machine_hive_and_keys
 
 
-def _get_user_keys():
-    '''
-    This will return the hive 'const' value and some registry keys where
-    installed software information has been known to exist for the
-    HKEY_USERS hive
-    '''
-    user_hive_and_keys = {}
-    user_keys = []
-    users_hive = win32con.HKEY_USERS
-    #skip some built in and default users since software information in these
-    #keys is limited
-    skip_users = ['.DEFAULT',
-                  'S-1-5-18',
-                  'S-1-5-19',
-                  'S-1-5-20']
-    sw_uninst_key = "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall"
-    reg_handle = win32api.RegOpenKeyEx(
-        users_hive,
-        '',
-        0,
-        win32con.KEY_READ)
-    for name, num, blank, time in win32api.RegEnumKeyEx(reg_handle):
-        #this is some identical key of a sid that contains some software names
-        #but no detailed information about the software installed for that user
-        if '_Classes' in name:
-            break
-        if name not in skip_users:
-            usr_sw_uninst_key = "\\".join([name, sw_uninst_key])
-            user_keys.append(usr_sw_uninst_key)
-    user_hive_and_keys[users_hive] = user_keys
-    return user_hive_and_keys
-
-
 def _get_reg_value(reg_hive, reg_key, value_name=''):
     '''
     Read one value from Windows registry.
@@ -483,21 +380,41 @@ def refresh_db(saltenv='base'):
     if not cached_repo:
         # It's not cached. Cache it, mate.
         cached_repo = __salt__['cp.cache_file'](repocache, saltenv)
+        if not cached_repo:
+            return False
         return True
     # Check if the master's cache file has changed
     if __salt__['cp.hash_file'](repocache) != __salt__['cp.hash_file'](cached_repo, saltenv):
         cached_repo = __salt__['cp.cache_file'](repocache, saltenv)
+        if not cached_repo:
+            return False
     return True
 
 
 def install(name=None, refresh=False, pkgs=None, saltenv='base', **kwargs):
     '''
-    Install the passed package
+    Install the passed package from the winrepo
 
-    Return a dict containing the new package names and versions::
+    :param name: The name of the package to install
+    :type name: str or None
 
-        {'<package>': {'old': '<old-version>',
-                       'new': '<new-version>'}}
+    :param bool refresh: Boolean value representing whether or not to refresh
+        the winrepo db
+
+    :param pkgs: A list of packages to install from a software repository.
+        All packages listed under ``pkgs`` will be installed via a single
+        command.
+    :type pkgs: list or None
+
+    :param str saltenv: The salt environment to use. Default is ``base``.
+
+    :param dict kwargs: Any additional argument that may be passed from the
+        state module. If they don't apply, they are ignored.
+
+    :return: Return a dict containing the new package names and versions::
+
+            {'<package>': {'old': '<old-version>',
+                           'new': '<new-version>'}}
 
     CLI Example:
 
@@ -508,7 +425,7 @@ def install(name=None, refresh=False, pkgs=None, saltenv='base', **kwargs):
     if salt.utils.is_true(refresh):
         refresh_db()
 
-    # Ignore pkg_type from parse_targets, Windows does not suport the "sources"
+    # Ignore pkg_type from parse_targets, Windows does not support the "sources"
     # argument
     pkg_params = __salt__['pkg_resource.parse_targets'](name,
                                                         pkgs,
@@ -535,9 +452,15 @@ def install(name=None, refresh=False, pkgs=None, saltenv='base', **kwargs):
 
         version_num = options and options.get('version') or _get_latest_pkg_version(pkginfo)
 
-        if version_num in [old.get(pkginfo[x]['full_name']) for x in pkginfo]:
+        if version_num in [old.get(pkg_name) for x in pkginfo]:
             # Desired version number already installed
-            continue
+            log.info('Version {0} is already installed for '
+                     'package {1}'.format(version_num, pkg_name))
+            return {
+                pkg_name: {
+                    'old': '{0} already installed'.format(version_num)
+                }
+            }
         elif version_num not in pkginfo:
             log.error('Version {0} not found for package '
                       '{1}'.format(version_num, pkg_name))
@@ -555,7 +478,13 @@ def install(name=None, refresh=False, pkgs=None, saltenv='base', **kwargs):
             cached_pkg = __salt__['cp.is_cached'](installer, saltenv)
             if not cached_pkg:
                 # It's not cached. Cache it, mate.
-                cached_pkg = __salt__['cp.cache_file'](installer, saltenv)
+                try:
+                    cached_pkg = __salt__['cp.cache_file'](installer, saltenv)
+                except MinionError as exc:
+                    return '{0}: {1}'.format(exc, installer)
+            if not cached_pkg:
+                return 'Unable to cache file {0} from saltenv: {1}'\
+                    .format(installer, saltenv)
             if __salt__['cp.hash_file'](installer, saltenv) != \
                                           __salt__['cp.hash_file'](cached_pkg):
                 cached_pkg = __salt__['cp.cache_file'](installer, saltenv)
@@ -564,17 +493,21 @@ def install(name=None, refresh=False, pkgs=None, saltenv='base', **kwargs):
 
         cached_pkg = cached_pkg.replace('/', '\\')
         msiexec = pkginfo[version_num].get('msiexec')
+        allusers = pkginfo[version_num].get('allusers')
+        if allusers is None:
+            allusers = True
         install_flags = '{0} {1}'.format(pkginfo[version_num]['install_flags'], options and options.get('extra_install_flags') or "")
 
         cmd = []
         if msiexec:
-            cmd.extend('msiexec', '/i')
+            cmd.extend(['msiexec', '/i'])
         cmd.append(cached_pkg)
         cmd.extend(install_flags.split())
+        if msiexec and allusers:
+            cmd.append('ALLUSERS="1"')
 
         __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
 
-    __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     return salt.utils.compare_dicts(old, new)
 
@@ -651,7 +584,10 @@ def remove(name=None, pkgs=None, version=None, extra_uninstall_flags=None, **kwa
             uninstaller = pkginfo[version].get('installer')
         if not uninstaller:
             return 'Error: No installer or uninstaller configured for package {0}'.format(name)
-        if uninstaller.startswith('salt:'):
+        if uninstaller.startswith('salt:') \
+                or uninstaller.startswith('http:') \
+                or uninstaller.startswith('https:') \
+                or uninstaller.startswith('ftp:'):
             cached_pkg = \
                 __salt__['cp.is_cached'](uninstaller)
             if not cached_pkg:
@@ -678,7 +614,6 @@ def remove(name=None, pkgs=None, version=None, extra_uninstall_flags=None, **kwa
 
         __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
 
-    __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     return salt.utils.compare_dicts(old, new)
 
@@ -735,6 +670,10 @@ def get_repo_data(saltenv='base'):
     cached_repo = __salt__['cp.is_cached'](repocache, saltenv)
     if not cached_repo:
         __salt__['pkg.refresh_db']()
+        cached_repo = __salt__['cp.is_cached'](repocache, saltenv)
+    if not __salt__['file.file_exists'](cached_repo):
+        log.error('No repo file found on the minion')
+        return {}
     try:
         with salt.utils.fopen(cached_repo, 'rb') as repofile:
             try:
@@ -754,7 +693,11 @@ def _get_name_map():
     '''
     Return a reverse map of full pkg names to the names recognized by winrepo.
     '''
-    return get_repo_data().get('name_map', {})
+    u_name_map = {}
+    name_map = get_repo_data().get('name_map', {})
+    for k in name_map.keys():
+        u_name_map[k.decode('utf-8')] = name_map[k]
+    return u_name_map
 
 
 def _get_package_info(name):
