@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 '''
 Manage the shadow file
+
+.. important::
+    If you feel that Salt should be using this module to manage passwords on a
+    minion, and it is using a different module (or gives an error similar to
+    *'shadow.info' is not available*), see :ref:`here
+    <module-provider-override>`.
 '''
+from __future__ import absolute_import
 
 import salt.utils
 
