@@ -2,14 +2,17 @@
 '''
 Define some generic socket functions for network modules
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import socket
-import subprocess
+import shlex
 import re
 import logging
 import os
 from string import ascii_letters, digits
+from salt.ext.six.moves import range
+import salt.ext.six as six
 
 # Attempt to import wmi
 try:
@@ -20,6 +23,7 @@ except ImportError:
 
 # Import salt libs
 import salt.utils
+from salt._compat import subprocess
 
 
 log = logging.getLogger(__name__)
@@ -398,6 +402,7 @@ def _interfaces_ip(out):
         based on the current set of cols
         '''
         brd = None
+        scope = None
         if '/' in value:  # we have a CIDR in this address
             ip, cidr = value.split('/')  # pylint: disable=C0103
         else:
@@ -410,7 +415,9 @@ def _interfaces_ip(out):
                 brd = cols[cols.index('brd') + 1]
         elif type_ == 'inet6':
             mask = cidr
-        return (ip, mask, brd)
+            if 'scope' in cols:
+                scope = cols[cols.index('scope') + 1]
+        return (ip, mask, brd, scope)
 
     groups = re.compile('\r?\n\\d').split(out)
     for group in groups:
@@ -437,7 +444,7 @@ def _interfaces_ip(out):
                 iflabel = cols[-1:][0]
                 if type_ in ('inet', 'inet6'):
                     if 'secondary' not in cols:
-                        ipaddr, netmask, broadcast = parse_network(value, cols)
+                        ipaddr, netmask, broadcast, scope = parse_network(value, cols)
                         if type_ == 'inet':
                             if 'inet' not in data:
                                 data['inet'] = list()
@@ -453,11 +460,12 @@ def _interfaces_ip(out):
                             addr_obj = dict()
                             addr_obj['address'] = ipaddr
                             addr_obj['prefixlen'] = netmask
+                            addr_obj['scope'] = scope
                             data['inet6'].append(addr_obj)
                     else:
                         if 'secondary' not in data:
                             data['secondary'] = list()
-                        ip_, mask, brd = parse_network(value, cols)
+                        ip_, mask, brd, scp = parse_network(value, cols)
                         data['secondary'].append({
                             'type': type_,
                             'address': ip_,
@@ -465,7 +473,7 @@ def _interfaces_ip(out):
                             'broadcast': brd,
                             'label': iflabel,
                         })
-                        del ip_, mask, brd
+                        del ip_, mask, brd, scp
                 elif type_.startswith('link'):
                     data['hwaddr'] = value
         if iface:
@@ -490,7 +498,7 @@ def _interfaces_ifconfig(out):
     else:
         pip = re.compile(r'.*?(?:inet addr:|inet )(.*?)\s')
         pip6 = re.compile('.*?(?:inet6 addr: (.*?)/|inet6 )([0-9a-fA-F:]+)')
-        pmask6 = re.compile(r'.*?(?:inet6 addr: [0-9a-fA-F:]+/(\d+)|prefixlen (\d+)).*')
+        pmask6 = re.compile(r'.*?(?:inet6 addr: [0-9a-fA-F:]+/(\d+)|prefixlen (\d+))(?: Scope:([a-zA-Z]+)| scopeid (0x[0-9a-fA-F]))?')
     pmask = re.compile(r'.*?(?:Mask:|netmask )(?:((?:0x)?[0-9a-fA-F]{8})|([\d\.]+))')
     pupdown = re.compile('UP')
     pbcast = re.compile(r'.*?(?:Bcast:|broadcast )([\d\.]+)')
@@ -510,6 +518,11 @@ def _interfaces_ifconfig(out):
                 iface = miface.group(1)
             if mmac:
                 data['hwaddr'] = mmac.group(1)
+                if salt.utils.is_sunos():
+                    expand_mac = []
+                    for chunk in data['hwaddr'].split(':'):
+                        expand_mac.append('0{0}'.format(chunk) if len(chunk) < 2 else '{0}'.format(chunk))
+                    data['hwaddr'] = ':'.join(expand_mac)
             if mip:
                 if 'inet' not in data:
                     data['inet'] = list()
@@ -537,6 +550,9 @@ def _interfaces_ifconfig(out):
                 mmask6 = pmask6.match(line)
                 if mmask6:
                     addr_obj['prefixlen'] = mmask6.group(1) or mmask6.group(2)
+                    if not salt.utils.is_sunos():
+                        ipv6scope = mmask6.group(3) or mmask6.group(4)
+                        addr_obj['scope'] = ipv6scope.lower() if ipv6scope is not None else ipv6scope
                 data['inet6'].append(addr_obj)
         data['up'] = updown
         if iface in ret:
@@ -717,25 +733,56 @@ def _ipv4_to_bits(ipaddr):
     return ''.join([bin(int(x))[2:].rjust(8, '0') for x in ipaddr.split('.')])
 
 
+def _get_iface_info(iface):
+    '''
+    If `iface` is available, return interface info and no error, otherwise
+    return no info and log and return an error
+    '''
+    iface_info = interfaces()
+
+    if iface in iface_info.keys():
+        return iface_info, False
+    else:
+        error_msg = ('Interface "{0}" not in available interfaces: "{1}"'
+                     ''.format(iface, '", "'.join(iface_info.keys())))
+        log.error(error_msg)
+        return None, error_msg
+
+
 def hw_addr(iface):
     '''
     Return the hardware address (a.k.a. MAC address) for a given interface
     '''
-    return interfaces().get(iface, {}).get('hwaddr', '')
+    iface_info, error = _get_iface_info(iface)
+
+    if error is False:
+        return iface_info.get(iface, {}).get('hwaddr', '')
+    else:
+        return error
 
 
 def interface(iface):
     '''
-    Return the the interface details
+    Return the details of `iface` or an error if it does not exist
     '''
-    return interfaces().get(iface, {}).get('inet', '')
+    iface_info, error = _get_iface_info(iface)
+
+    if error is False:
+        return iface_info.get(iface, {}).get('inet', '')
+    else:
+        return error
 
 
 def interface_ip(iface):
     '''
-    Return the the interface details
+    Return `iface` IPv4 addr or an error if `iface` does not exist
     '''
-    return interfaces().get(iface, {}).get('inet', {})[0].get('address', {})
+    iface_info, error = _get_iface_info(iface)
+
+    if error is False:
+        return iface_info.get(iface, {}).get('inet', {})[0].get('address', '')
+    else:
+        return error
 
 
 def subnets():
@@ -745,7 +792,7 @@ def subnets():
     ifaces = interfaces()
     subnetworks = []
 
-    for ipv4_info in ifaces.itervalues():
+    for ipv4_info in six.itervalues(ifaces):
         for ipv4 in ipv4_info.get('inet', []):
             if ipv4['address'] == '127.0.0.1':
                 continue
@@ -788,22 +835,35 @@ def in_subnet(cidr, addrs=None):
     return False
 
 
-def ip_addrs(interface=None, include_loopback=False):
+def ip_in_subnet(ip_addr, cidr):
+    '''
+    Returns True if given IP is within specified subnet, otherwise False
+    '''
+    ipaddr = int(''.join(['%02x' % int(x) for x in ip_addr.split('.')]), 16)  # pylint: disable=E1321
+    netstr, bits = cidr.split('/')
+    netaddr = int(''.join(['%02x' % int(x) for x in netstr.split('.')]), 16)  # pylint: disable=E1321
+    mask = (0xffffffff << (32 - int(bits))) & 0xffffffff
+    return (ipaddr & mask) == (netaddr & mask)
+
+
+def ip_addrs(interface=None, include_loopback=False, interface_data=None):
     '''
     Returns a list of IPv4 addresses assigned to the host. 127.0.0.1 is
     ignored, unless 'include_loopback=True' is indicated. If 'interface' is
     provided, then only IP addresses from that interface will be returned.
     '''
     ret = set()
-    ifaces = interfaces()
+    ifaces = interface_data \
+        if isinstance(interface_data, dict) \
+        else interfaces()
     if interface is None:
         target_ifaces = ifaces
     else:
-        target_ifaces = dict([(k, v) for k, v in ifaces.iteritems()
+        target_ifaces = dict([(k, v) for k, v in six.iteritems(ifaces)
                               if k == interface])
         if not target_ifaces:
             log.error('Interface {0} not found.'.format(interface))
-    for ipv4_info in target_ifaces.itervalues():
+    for ipv4_info in six.itervalues(target_ifaces):
         for ipv4 in ipv4_info.get('inet', []):
             loopback = in_subnet('127.0.0.0/8', [ipv4.get('address')]) or ipv4.get('label') == 'lo'
             if not loopback or include_loopback:
@@ -816,22 +876,24 @@ def ip_addrs(interface=None, include_loopback=False):
     return sorted(list(ret))
 
 
-def ip_addrs6(interface=None, include_loopback=False):
+def ip_addrs6(interface=None, include_loopback=False, interface_data=None):
     '''
     Returns a list of IPv6 addresses assigned to the host. ::1 is ignored,
     unless 'include_loopback=True' is indicated. If 'interface' is provided,
     then only IP addresses from that interface will be returned.
     '''
     ret = set()
-    ifaces = interfaces()
+    ifaces = interface_data \
+        if isinstance(interface_data, dict) \
+        else interfaces()
     if interface is None:
         target_ifaces = ifaces
     else:
-        target_ifaces = dict([(k, v) for k, v in ifaces.iteritems()
+        target_ifaces = dict([(k, v) for k, v in six.iteritems(ifaces)
                               if k == interface])
         if not target_ifaces:
             log.error('Interface {0} not found.'.format(interface))
-    for ipv6_info in target_ifaces.itervalues():
+    for ipv6_info in six.itervalues(target_ifaces):
         for ipv6 in ipv6_info.get('inet6', []):
             if include_loopback or ipv6['address'] != '::1':
                 ret.add(ipv6['address'])
@@ -889,7 +951,7 @@ def local_port_tcp(port):
                 if line.strip().startswith('sl'):
                     continue
                 iret = _parse_tcp_line(line)
-                sl = iter(iret).next()
+                sl = next(iter(iret))
                 if iret[sl]['local_port'] == port:
                     ret.add(iret[sl]['remote_addr'])
         return ret
@@ -909,7 +971,7 @@ def remote_port_tcp(port):
                 if line.strip().startswith('sl'):
                     continue
                 iret = _parse_tcp_line(line)
-                sl = iter(iret).next()
+                sl = next(iter(iret))
                 if iret[sl]['remote_port'] == port:
                     ret.add(iret[sl]['remote_addr'])
         return ret
@@ -952,7 +1014,7 @@ def _sunos_remotes_on(port, which_end):
     '''
     remotes = set()
     try:
-        data = subprocess.check_output(['netstat', '-f', 'inet', '-n'])
+        data = subprocess.check_output(['netstat', '-f', 'inet', '-n'])  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError:
         log.error('Failed netstat')
         raise
@@ -997,7 +1059,8 @@ def _freebsd_remotes_on(port, which_end):
     remotes = set()
 
     try:
-        data = subprocess.check_output(['sockstat', '-4', '-c', '-p {0}'.format(port)])
+        cmd = shlex.split('sockstat -4 -c -p {0}'.format(port))
+        data = subprocess.check_output(cmd)  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "sockstat" with returncode = {0}'.format(ex.returncode))
         raise
@@ -1053,7 +1116,7 @@ def remotes_on_local_tcp_port(port):
         return _freebsd_remotes_on(port, 'local_port')
 
     try:
-        data = subprocess.check_output(['lsof', '-i4TCP:{0:d}'.format(port), '-n'])
+        data = subprocess.check_output(['lsof', '-i4TCP:{0:d}'.format(port), '-n'])  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "lsof" with returncode = {0}'.format(ex.returncode))
         raise
@@ -1104,7 +1167,7 @@ def remotes_on_remote_tcp_port(port):
         return _freebsd_remotes_on(port, 'remote_port')
 
     try:
-        data = subprocess.check_output(['lsof', '-i4TCP:{0:d}'.format(port), '-n'])
+        data = subprocess.check_output(['lsof', '-i4TCP:{0:d}'.format(port), '-n'])  # pylint: disable=minimum-python-version
     except subprocess.CalledProcessError as ex:
         log.error('Failed "lsof" with returncode = {0}'.format(ex.returncode))
         raise
@@ -1180,3 +1243,10 @@ class IPv4Address(object):
         :return: True if the address is a loopback address. Otherwise False.
         '''
         return 127 == self.dotted_quad[0]
+
+    @property
+    def reverse_pointer(self):
+        '''
+        :return: Reversed IP address
+        '''
+        return '.'.join(reversed(self.dotted_quad)) + '.in-addr.arpa.'
