@@ -5,6 +5,8 @@ Tests for the file state
 '''
 
 # Import python libs
+from __future__ import absolute_import
+from distutils.version import LooseVersion
 import glob
 import grp
 import os
@@ -12,6 +14,7 @@ import pwd
 import shutil
 import stat
 import tempfile
+import textwrap
 import filecmp
 
 # Import Salt Testing libs
@@ -21,13 +24,28 @@ from salttesting.helpers import (
     ensure_in_syspath,
     with_system_user_and_group
 )
-ensure_in_syspath('../../')
 
+ensure_in_syspath('../../')
 
 # Import salt libs
 import integration
 import salt.utils
 
+GIT_PYTHON = '0.3.2'
+HAS_GIT_PYTHON = False
+
+try:
+    import git
+    if LooseVersion(git.__version__) >= LooseVersion(GIT_PYTHON):
+        HAS_GIT_PYTHON = True
+except ImportError:
+    HAS_GIT_PYTHON = False
+
+STATE_DIR = os.path.join(integration.FILES, 'file', 'base')
+FILEPILLAR = '/tmp/filepillar-python'
+FILEPILLARDEF = '/tmp/filepillar-defaultvalue'
+FILEPILLARGIT = '/tmp/filepillar-bar'
+
 
 class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
     '''
@@ -184,6 +202,50 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         self.assertEqual(oct(desired_mode), oct(resulting_mode))
         self.assertSaltTrueReturn(ret)
 
+    @destructiveTest
+    def test_managed_file_with_pillar_sls(self):
+        '''
+        Test to ensure pillar data in sls file
+        is rendered properly and file is created.
+        '''
+        state_name = 'file-pillarget'
+        ret = self.run_function('state.sls', [state_name])
+        self.assertSaltTrueReturn(ret)
+
+        #Check to make sure the file was created
+        check_file = self.run_function('file.file_exists', [FILEPILLAR])
+        self.assertTrue(check_file)
+
+    @destructiveTest
+    def test_managed_file_with_pillardefault_sls(self):
+        '''
+        Test to ensure when pillar data is not available
+        in sls file with pillar.get it uses the default
+        value.
+        '''
+        state_name = 'file-pillardefaultget'
+        ret = self.run_function('state.sls', [state_name])
+        self.assertSaltTrueReturn(ret)
+
+        #Check to make sure the file was created
+        check_file = self.run_function('file.file_exists', [FILEPILLARDEF])
+        self.assertTrue(check_file)
+
+    @skipIf(not HAS_GIT_PYTHON, "GitFS could not be loaded. Skipping test")
+    @destructiveTest
+    def test_managed_file_with_gitpillar_sls(self):
+        '''
+        Test to ensure git pillar data in sls
+        file is rendered properly and is created.
+        '''
+        state_name = 'file-pillargit'
+        ret = self.run_function('state.sls', [state_name])
+        self.assertSaltTrueReturn(ret)
+
+        #Check to make sure the file was created
+        check_file = self.run_function('file.file_exists', [FILEPILLARGIT])
+        self.assertTrue(check_file)
+
     @skipIf(os.geteuid() != 0, 'you must be root to run this test')
     def test_managed_dir_mode(self):
         '''
@@ -237,6 +299,65 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         changes = ret.values()[0]['changes']
         self.assertEqual('<show_diff=False>', changes['diff'])
 
+    def test_managed_contents(self):
+        '''
+        test file.managed with contents that is a boolean, string, integer,
+        float, list, and dictionary
+        '''
+        state_name = 'file-FileTest-test_managed_contents'
+        state_filename = state_name + '.sls'
+        state_file = os.path.join(STATE_DIR, state_filename)
+
+        managed_files = {}
+        state_keys = {}
+        for typ in ('bool', 'str', 'int', 'float', 'list', 'dict'):
+            managed_files[typ] = tempfile.mkstemp()[1]
+            state_keys[typ] = 'file_|-{0} file_|-{1}_|-managed'.format(typ, managed_files[typ])
+        try:
+            salt.utils.fopen(state_file, 'w').write(textwrap.dedent('''\
+                bool file:
+                  file.managed:
+                    - name: {bool}
+                    - contents: True
+
+                str file:
+                  file.managed:
+                    - name: {str}
+                    - contents: Salt was here.
+
+                int file:
+                  file.managed:
+                    - name: {int}
+                    - contents: 340282366920938463463374607431768211456
+
+                float file:
+                  file.managed:
+                    - name: {float}
+                    - contents: 1.7518e-45  # gravitational coupling constant
+
+                list file:
+                  file.managed:
+                    - name: {list}
+                    - contents: [1, 1, 2, 3, 5, 8, 13]
+
+                dict file:
+                  file.managed:
+                    - name: {dict}
+                    - contents:
+                        C: charge
+                        P: parity
+                        T: time
+                '''.format(**managed_files)))
+
+            ret = self.run_function('state.sls', [state_name])
+            for typ in state_keys:
+                self.assertTrue(ret[state_keys[typ]]['result'])
+                self.assertIn('diff', ret[state_keys[typ]]['changes'])
+        finally:
+            os.remove(state_file)
+            for typ in managed_files:
+                os.remove(managed_files[typ])
+
     def test_directory(self):
         '''
         file.directory
@@ -856,40 +977,6 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         finally:
             os.remove(path_test)
 
-    def test_sed(self):
-        '''
-        file.sed
-        '''
-        name = os.path.join(integration.TMP, 'sed_test')
-        with salt.utils.fopen(name, 'w+') as fp_:
-            fp_.write('change_me')
-        ret = self.run_state(
-            'file.sed', name=name, before='change', after='salt'
-        )
-        try:
-            with salt.utils.fopen(name, 'r') as fp_:
-                self.assertIn('salt', fp_.read())
-            self.assertSaltTrueReturn(ret)
-        finally:
-            os.remove(name)
-
-    def test_test_sed(self):
-        '''
-        file.sed test integration
-        '''
-        name = os.path.join(integration.TMP, 'sed_test_test')
-        with salt.utils.fopen(name, 'w+') as fp_:
-            fp_.write('change_me')
-        ret = self.run_state(
-            'file.sed', test=True, name=name, before='change', after='salt'
-        )
-        try:
-            with salt.utils.fopen(name, 'r') as fp_:
-                self.assertIn('change', fp_.read())
-            self.assertSaltNoneReturn(ret)
-        finally:
-            os.remove(name)
-
     def test_comment(self):
         '''
         file.comment
@@ -1028,6 +1115,17 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
                 'file.append', name=name, text='cheese', makedirs=True
             )
             self.assertSaltTrueReturn(ret)
+        finally:
+            if os.path.isfile(name):
+                os.remove(name)
+
+        try:
+            # Parent directory exists but file does not and makedirs is False
+            ret = self.run_state(
+                'file.append', name=name, text='cheese'
+            )
+            self.assertSaltTrueReturn(ret)
+            self.assertTrue(os.path.isfile(name))
         finally:
             shutil.rmtree(
                 os.path.join(integration.TMP, 'issue_1864'),
@@ -1688,6 +1786,33 @@ class FileTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
         finally:
             os.remove(source)
 
+    def test_issue_25250_force_copy_deletes(self):
+        '''
+        ensure force option in copy state does not delete target file
+        '''
+        dest = os.path.join(integration.TMP, 'dest')
+        source = os.path.join(integration.TMP, 'source')
+        shutil.copyfile(os.path.join(integration.FILES, 'hosts'), source)
+        shutil.copyfile(os.path.join(integration.FILES, 'file/base/cheese'), dest)
+
+        self.run_state('file.copy', name=dest, source=source, force=True)
+        self.assertTrue(os.path.exists(dest))
+        self.assertTrue(filecmp.cmp(source, dest))
+
+        os.remove(source)
+        os.remove(dest)
+
+    @destructiveTest
+    def tearDown(self):
+        '''
+        remove files created in previous tests
+        '''
+        all_files = [FILEPILLAR, FILEPILLARDEF, FILEPILLARGIT]
+        for file in all_files:
+            check_file = self.run_function('file.file_exists', [file])
+            if check_file:
+                self.run_function('file.remove', [file])
+
 if __name__ == '__main__':
     from integration import run_tests
     run_tests(FileTest)
